import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;

/*usage: java FindSimilarAppsNoObf <app package in dataset> (optional: <number of top apps to display> optional: "-sc" (displays results using only same category))
requires a list of apps with their respective category, separated by a comma (e.g., com.rovio.angrybirds,ARCADE)

This program assumes you have different similarity files for a dataset without obfuscated apps

examples:

This returns the similar apps that are found in the goldset for com.rovio.angrybirds:
java FindSimilarAppsNoObf com.rovio.angrybirds

This returns the similar apps that are found in the goldset for com.rovio.angrybirds, as well as the top 3 apps:
java FindSimilarAppsNoObf com.rovio.angrybirds 3

This returns the similar apps that are found in the goldset for com.rovio.angrybirds, ranking only apps of the same category:
java FindSimilarAppsNoObf com.rovio.angrybirds 0 -sc

This returns the similar apps that are found in the goldset for com.rovio.angrybirds, as well as the top 2 apps, ranking only apps of the same category:
java FindSimilarAppsNoObf com.rovio.angrybirds 2 -sc

*/

public class FindSimilarAppsNoObf {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		try{ 
			int topcount = 0; //extra argument that allows printing of top X apps as specified by argument[1]
			boolean comparemode = false;
			boolean samecatmode = false;
			if(args.length==1){
				
			}
			else if(args[1].matches("-?\\d+")){ //we have an integer
				topcount = Integer.valueOf(args[1]);
				if(args.length==3){
					if(args[2].equals("-sc")){
						samecatmode = true;
					}
				}
			}
			else{
				comparemode = true;
				//System.out.println("Please make sure the second argument is either an integer or is empty.");
			}
			
			ArrayList<String> appnames = new ArrayList<String>(); //list of apps we have in dataset
			ArrayList<String> appcats = new ArrayList<String>(); //category for each app
		
			File applist = new File("//data//similarities//app_list_cats_noobf.txt");
			//File applist = new File("//scratch//asholtzh//app_list.txt");
			BufferedReader in = new BufferedReader(new FileReader(applist));
			String buf;
			while((buf=in.readLine())!=null){
				String[] currline = buf.split(";");
				appnames.add(currline[0]);
				appcats.add(currline[1]);
			}
			in.close();
			
			//list of apps filled in
		
			String inputapp = args[0];
			String inputcat = appcats.get(appnames.indexOf(inputapp));
			if(appnames.contains(inputapp)){
				int compareidx = 0;
				if(comparemode){
					String compareapp = args[1];
					compareidx = appnames.indexOf(compareapp)+1;
					if(!appnames.contains(compareapp)){
						System.out.println("Provided second app package name not in dataset.  Make the sure the app package name is spelled correctly.");
						System.exit(1);
					}
				}
				//int appfindnum = appnames.indexOf(inputapp)+1; //the id of the app (used when checking similarity files)
				File goldset = new File("//data//similarities//goldsets//" + inputapp + ".txt");
				//File goldset = new File("//scratch//asholtzh//related_apps//" + inputapp + ".txt");
				BufferedReader goldin = new BufferedReader(new FileReader(goldset));
				String goldbuf;
				boolean foundapp = false; //used to check if at least one app in goldset is in dataset, otherwise exit
				boolean oneapp = false; //used to see if the app has any related apps at all
				ArrayList<Integer> relappids = new ArrayList<Integer>(); //used to keep the app ids of the related apps, need to find these later in the similarity file
				System.out.println("---------RELATED APPS---------");
				while((goldbuf=goldin.readLine()) != null){
					if(appnames.contains(goldbuf)){
						foundapp = true;
						relappids.add(appnames.indexOf(goldbuf)+1); //need the +1 as similarities start with 1, not index 0
						System.out.println(goldbuf);
					}
					oneapp = true;
				}
				System.out.println("------------------------------");
				goldin.close();

				if(!oneapp){
					System.out.println("There are no related apps for this app, please choose another app.");
					System.exit(1);
				}				
				if(!foundapp){
					System.out.println("None of the related apps for this app are in our dataset, please choose another app.");
					System.exit(1);
				}
				
				
				//API
				File api_sim = new File("//data//similarities//apis_noobf//"+inputapp+".txt");
				BufferedReader api_in = new BufferedReader(new FileReader(api_sim));
				String api_buf;
				ArrayList<AppSim> apiranklist = new ArrayList<AppSim>();
				
				while((api_buf = api_in.readLine()) != null){
					String[] linesplit = api_buf.split("\\s+");
					int firstapp = Integer.valueOf(linesplit[0]);
					int secondapp = Integer.valueOf(linesplit[1]);
					
					if(firstapp == secondapp){ //comparison to self, skip this line
						continue;
					}
					
					DecimalFormat formatter = new DecimalFormat("0.00000000000");
					String tempval = formatter.format(Double.valueOf(linesplit[2]));
					double simval = Double.valueOf(tempval);
					
					if(!samecatmode){ //not checking for same categories, add every app
						apiranklist.add(new AppSim(simval, secondapp));
					} else{
						if(appcats.get(secondapp-1).equals(inputcat)){ //if the apps are the same category
							apiranklist.add(new AppSim(simval, secondapp));	
						}
					}
				}
				api_in.close();
				//END API
				
				//IDENTIFIERS
				File id_sim = new File("//data//similarities//identifiers_noobf//"+inputapp+".txt");
				BufferedReader id_in = new BufferedReader(new FileReader(id_sim));
				String id_buf;
				ArrayList<AppSim> idranklist = new ArrayList<AppSim>();
				
				while((id_buf = id_in.readLine()) != null){
					String[] linesplit = id_buf.split("\\s+");
					int firstapp = Integer.valueOf(linesplit[0]);
					int secondapp = Integer.valueOf(linesplit[1]);
					
					if(firstapp == secondapp){ //comparison to self, skip this line
						continue;
					}
					
					DecimalFormat formatter = new DecimalFormat("0.00000000000");
					String tempval = formatter.format(Double.valueOf(linesplit[2]));
					double simval = Double.valueOf(tempval);
					if(!samecatmode){ //not checking for same categories, add every app
						idranklist.add(new AppSim(simval, secondapp));
					} else{
						if(appcats.get(secondapp-1).equals(inputcat)){ //if the apps are the same category
							idranklist.add(new AppSim(simval, secondapp));	
						}
					}
				}
				id_in.close();
				//END IDENTIFIERS
				
				//INTENTS
				File intent_sim = new File("//data//similarities//intents_noobf//"+inputapp+".txt");
				BufferedReader intent_in = new BufferedReader(new FileReader(intent_sim));
				String intent_buf;
				ArrayList<AppSim> intentranklist = new ArrayList<AppSim>();
				
				while((intent_buf = intent_in.readLine()) != null){
					String[] linesplit = intent_buf.split("\\s+");
					int firstapp = Integer.valueOf(linesplit[0]);
					int secondapp = Integer.valueOf(linesplit[1]);
					
					if(firstapp == secondapp){ //comparison to self, skip this line
						continue;
					}
					
					DecimalFormat formatter = new DecimalFormat("0.00000000000");
					String tempval = formatter.format(Double.valueOf(linesplit[2]));
					double simval = Double.valueOf(tempval);
					if(!samecatmode){ //not checking for same categories, add every app
						intentranklist.add(new AppSim(simval, secondapp));
					} else{
						if(appcats.get(secondapp-1).equals(inputcat)){ //if the apps are the same category
							intentranklist.add(new AppSim(simval, secondapp));	
						}
					}
				}
				intent_in.close();
				//END INTENTS
				
				//PERMISSIONS
				File perm_sim = new File("//data//similarities//permissions_noobf//"+inputapp+".txt");
				BufferedReader perm_in = new BufferedReader(new FileReader(perm_sim));
				String perm_buf;
				ArrayList<AppSim> permranklist = new ArrayList<AppSim>();
				
				while((perm_buf = perm_in.readLine()) != null){
					String[] linesplit = perm_buf.split("\\s+");
					int firstapp = Integer.valueOf(linesplit[0]);
					int secondapp = Integer.valueOf(linesplit[1]);
					
					if(firstapp == secondapp){ //comparison to self, skip this line
						continue;
					}
					
					DecimalFormat formatter = new DecimalFormat("0.00000000000");
					String tempval = formatter.format(Double.valueOf(linesplit[2]));
					double simval = Double.valueOf(tempval);
					if(!samecatmode){ //not checking for same categories, add every app
						permranklist.add(new AppSim(simval, secondapp));
					} else{
						if(appcats.get(secondapp-1).equals(inputcat)){ //if the apps are the same category
							permranklist.add(new AppSim(simval, secondapp));	
						}
					}
				}
				perm_in.close();
				//END PERMISSIONS
				
				//SENSORS
				File sens_sim = new File("//data//similarities//sensors_noobf//"+inputapp+".txt");
				BufferedReader sens_in = new BufferedReader(new FileReader(sens_sim));
				String sens_buf;
				ArrayList<AppSim> sensranklist = new ArrayList<AppSim>();
				
				while((sens_buf = sens_in.readLine()) != null){
					String[] linesplit = sens_buf.split("\\s+");
					int firstapp = Integer.valueOf(linesplit[0]);
					int secondapp = Integer.valueOf(linesplit[1]);
					
					if(firstapp == secondapp){ //comparison to self, skip this line
						continue;
					}
					
					DecimalFormat formatter = new DecimalFormat("0.00000000000");
					String tempval = formatter.format(Double.valueOf(linesplit[2]));
					double simval = Double.valueOf(tempval);
					if(!samecatmode){ //not checking for same categories, add every app
						sensranklist.add(new AppSim(simval, secondapp));
					} else{
						if(appcats.get(secondapp-1).equals(inputcat)){ //if the apps are the same category
							sensranklist.add(new AppSim(simval, secondapp));	
						}
					}
				}
				sens_in.close();
				//END SENSORS
				
				//COMBINED API+IDENTIFIERS
				ArrayList<AppSim> combinedranklist = new ArrayList<AppSim>();
				int listcounter=0;
				for(AppSim app : apiranklist){
					double combval = (app.getVal() + idranklist.get(listcounter).getVal())/2;
					//combval += idranklist.get(listcounter).getVal();
					combinedranklist.add(new AppSim(combval, app.getID()));
					listcounter++;
				}
				//END COMBINED
				
				System.out.println("Sorting similarity values");
				
				
				Collections.sort(apiranklist, new Comparator<AppSim>(){ //sorts smallest to largest
				    public int compare(AppSim appval1, AppSim appval2) {
				        return Double.compare(appval1.getVal(), appval2.getVal());
				    }
				});
				Collections.reverse(apiranklist); //reverse for correct order
				
				Collections.sort(idranklist, new Comparator<AppSim>(){ //sorts smallest to largest
				    public int compare(AppSim appval1, AppSim appval2) {
				        return Double.compare(appval1.getVal(), appval2.getVal());
				    }
				});
				Collections.reverse(idranklist); //reverse for correct order
				
				Collections.sort(intentranklist, new Comparator<AppSim>(){ //sorts smallest to largest
				    public int compare(AppSim appval1, AppSim appval2) {
				        return Double.compare(appval1.getVal(), appval2.getVal());
				    }
				});
				Collections.reverse(intentranklist); //reverse for correct order
				
				Collections.sort(permranklist, new Comparator<AppSim>(){ //sorts smallest to largest
				    public int compare(AppSim appval1, AppSim appval2) {
				        return Double.compare(appval1.getVal(), appval2.getVal());
				    }
				});
				Collections.reverse(permranklist); //reverse for correct order
				
				Collections.sort(sensranklist, new Comparator<AppSim>(){ //sorts smallest to largest
				    public int compare(AppSim appval1, AppSim appval2) {
				        return Double.compare(appval1.getVal(), appval2.getVal());
				    }
				});
				Collections.reverse(sensranklist); //reverse for correct order
				
				Collections.sort(combinedranklist, new Comparator<AppSim>(){ //sorts smallest to largest
				    public int compare(AppSim appval1, AppSim appval2) {
				        return Double.compare(appval1.getVal(), appval2.getVal());
				    }
				});
				Collections.reverse(combinedranklist); //reverse for correct order
				
				int counter = 1;
				DecimalFormat valformat = new DecimalFormat("0.00000000000");
				
				//int avgcounter = 0;
				//double avg_api = 0;
				
				int prevrank = 0;
				String prevval = "";
				int countprint = 1;
				
				System.out.println("---------API RANKINGS---------");
				for(AppSim rankedapp : apiranklist){
					int tempid = rankedapp.getID();
					String val = valformat.format(Double.valueOf(rankedapp.getVal()));
					if(prevval.equals(val)){
						prevrank++;
						countprint = counter-prevrank;
					}
					else{
						prevrank = 0;
						countprint = counter;
					}
					prevval = val;
					if(!comparemode){
						if(topcount>0){
							if(counter<topcount+1){
								//String val = valformat.format(Double.valueOf(rankedapp.getVal()));
								System.out.println("Rank: " + countprint + ",\tVal: " + val + ",\tID: " + tempid + ",\tName: " + appnames.get(tempid-1)); //accommodate for extra 1	
							}
						}
						if(relappids.contains(tempid)){
							//String val = valformat.format(Double.valueOf(rankedapp.getVal()));
							System.out.println("Rank: " + countprint + ",\tVal: " + val + ",\tID: " + tempid + ",\tName: " + appnames.get(tempid-1)); //accommodate for extra 1
							//avgcounter++;
							//avg_api += counter;
						}
					} 
					else{
						if(tempid == compareidx){
							//String val = valformat.format(Double.valueOf(rankedapp.getVal()));
							System.out.println("Rank: " + countprint + ",\tVal: " + val + ",\tID: " + tempid + ",\tName: " + appnames.get(tempid-1)); //accommodate for extra 1
							break;
						}
					}
					counter++;
				}
				//System.out.println("Average ranking: " + (avg_api/avgcounter));
				System.out.println("------------------------------");
				

				
				counter=1;
				prevrank = 0;
				prevval = "";
				countprint = 1;
				System.out.println("-----IDENTIFIERS RANKINGS-----");
				for(AppSim rankedapp : idranklist){
					int tempid = rankedapp.getID();
					String val = valformat.format(Double.valueOf(rankedapp.getVal()));
					if(prevval.equals(val)){
						prevrank++;
						countprint = counter-prevrank;
					}
					else{
						prevrank = 0;
						countprint = counter;
					}
					prevval = val;
					if(!comparemode){
						if(topcount>0){
							if(counter<topcount+1){
								//String val = valformat.format(Double.valueOf(rankedapp.getVal()));
								System.out.println("Rank: " + countprint + ",\tVal: " + val + ",\tID: " + tempid + ",\tName: " + appnames.get(tempid-1)); //accommodate for extra 1	
							}
						}
						if(relappids.contains(tempid)){
							//String val = valformat.format(Double.valueOf(rankedapp.getVal()));
							System.out.println("Rank: " + countprint + ",\tVal: " + val + ",\tID: " + tempid + ",\tName: " + appnames.get(tempid-1)); //accommodate for extra 1
							//avgcounter++;
							//avg_api += counter;
						}
					} 
					else{
						if(tempid == compareidx){
							//String val = valformat.format(Double.valueOf(rankedapp.getVal()));
							System.out.println("Rank: " + countprint + ",\tVal: " + val + ",\tID: " + tempid + ",\tName: " + appnames.get(tempid-1)); //accommodate for extra 1
							break;
						}
					}
					
					counter++;
				}
				System.out.println("------------------------------");
				
				counter=1;
				prevrank = 0;
				prevval = "";
				countprint = 1;
				System.out.println("-------COMBINED RANKINGS------");
				for(AppSim rankedapp : combinedranklist){
					int tempid = rankedapp.getID();
					String val = valformat.format(Double.valueOf(rankedapp.getVal()));
					if(prevval.equals(val)){
						prevrank++;
						countprint = counter-prevrank;
					}
					else{
						prevrank = 0;
						countprint = counter;
					}
					prevval = val;
					if(!comparemode){
						if(topcount>0){
							if(counter<topcount+1){
								//String val = valformat.format(Double.valueOf(rankedapp.getVal()));
								System.out.println("Rank: " + countprint + ",\tVal: " + val + ",\tID: " + tempid + ",\tName: " + appnames.get(tempid-1)); //accommodate for extra 1	
							}
						}
						if(relappids.contains(tempid)){
							//String val = valformat.format(Double.valueOf(rankedapp.getVal()));
							System.out.println("Rank: " + countprint + ",\tVal: " + val + ",\tID: " + tempid + ",\tName: " + appnames.get(tempid-1)); //accommodate for extra 1
							//avgcounter++;
							//avg_api += counter;
						}
					} 
					else{
						if(tempid == compareidx){
							//String val = valformat.format(Double.valueOf(rankedapp.getVal()));
							System.out.println("Rank: " + countprint + ",\tVal: " + val + ",\tID: " + tempid + ",\tName: " + appnames.get(tempid-1)); //accommodate for extra 1
							break;
						}
					}
					counter++;
				}
				System.out.println("------------------------------");
				
				counter=1;
				prevrank = 0;
				prevval = "";
				countprint = 1;
				System.out.println("-------INTENTS RANKINGS-------");
				for(AppSim rankedapp : intentranklist){
					int tempid = rankedapp.getID();
					String val = valformat.format(Double.valueOf(rankedapp.getVal()));
					if(prevval.equals(val)){
						prevrank++;
						countprint = counter-prevrank;
					}
					else{
						prevrank = 0;
						countprint = counter;
					}
					prevval = val;
					if(!comparemode){
						if(topcount>0){
							if(counter<topcount+1){
								//String val = valformat.format(Double.valueOf(rankedapp.getVal()));
								System.out.println("Rank: " + countprint + ",\tVal: " + val + ",\tID: " + tempid + ",\tName: " + appnames.get(tempid-1)); //accommodate for extra 1	
							}
						}
						if(relappids.contains(tempid)){
							//String val = valformat.format(Double.valueOf(rankedapp.getVal()));
							System.out.println("Rank: " + countprint + ",\tVal: " + val + ",\tID: " + tempid + ",\tName: " + appnames.get(tempid-1)); //accommodate for extra 1
							//avgcounter++;
							//avg_api += counter;
						}
					} 
					else{
						if(tempid == compareidx){
							//String val = valformat.format(Double.valueOf(rankedapp.getVal()));
							System.out.println("Rank: " + countprint + ",\tVal: " + val + ",\tID: " + tempid + ",\tName: " + appnames.get(tempid-1)); //accommodate for extra 1
							break;
						}
					}
					counter++;
				}
				System.out.println("------------------------------");
				
				counter=1;
				prevrank = 0;
				prevval = "";
				countprint = 1;
				System.out.println("-----PERMISSIONS RANKINGS-----");
				for(AppSim rankedapp : permranklist){
					int tempid = rankedapp.getID();
					String val = valformat.format(Double.valueOf(rankedapp.getVal()));
					if(prevval.equals(val)){
						prevrank++;
						countprint = counter-prevrank;
					}
					else{
						prevrank = 0;
						countprint = counter;
					}
					prevval = val;
					if(!comparemode){
						if(topcount>0){
							if(counter<topcount+1){
								//String val = valformat.format(Double.valueOf(rankedapp.getVal()));
								System.out.println("Rank: " + countprint + ",\tVal: " + val + ",\tID: " + tempid + ",\tName: " + appnames.get(tempid-1)); //accommodate for extra 1	
							}
						}
						if(relappids.contains(tempid)){
							//String val = valformat.format(Double.valueOf(rankedapp.getVal()));
							System.out.println("Rank: " + countprint + ",\tVal: " + val + ",\tID: " + tempid + ",\tName: " + appnames.get(tempid-1)); //accommodate for extra 1
							//avgcounter++;
							//avg_api += counter;
						}
					} 
					else{
						if(tempid == compareidx){
							//String val = valformat.format(Double.valueOf(rankedapp.getVal()));
							System.out.println("Rank: " + countprint + ",\tVal: " + val + ",\tID: " + tempid + ",\tName: " + appnames.get(tempid-1)); //accommodate for extra 1
							break;
						}
					}
					counter++;
				}
				System.out.println("------------------------------");
				
				counter=1;
				prevrank = 0;
				prevval = "";
				countprint = 1;
				System.out.println("-------SENSORS RANKINGS-------");
				for(AppSim rankedapp : sensranklist){
					int tempid = rankedapp.getID();
					String val = valformat.format(Double.valueOf(rankedapp.getVal()));
					if(prevval.equals(val)){
						prevrank++;
						countprint = counter-prevrank;
					}
					else{
						prevrank = 0;
						countprint = counter;
					}
					prevval = val;
					if(!comparemode){
						if(topcount>0){
							if(counter<topcount+1){
								//String val = valformat.format(Double.valueOf(rankedapp.getVal()));
								System.out.println("Rank: " + countprint + ",\tVal: " + val + ",\tID: " + tempid + ",\tName: " + appnames.get(tempid-1)); //accommodate for extra 1	
							}
						}
						if(relappids.contains(tempid)){
							//String val = valformat.format(Double.valueOf(rankedapp.getVal()));
							System.out.println("Rank: " + countprint + ",\tVal: " + val + ",\tID: " + tempid + ",\tName: " + appnames.get(tempid-1)); //accommodate for extra 1
							//avgcounter++;
							//avg_api += counter;
						}
					} 
					else{
						if(tempid == compareidx){
							//String val = valformat.format(Double.valueOf(rankedapp.getVal()));
							System.out.println("Rank: " + countprint + ",\tVal: " + val + ",\tID: " + tempid + ",\tName: " + appnames.get(tempid-1)); //accommodate for extra 1
							break;
						}
					}
					counter++;
				}
				System.out.println("------------------------------");
				
				
				
			}else{
				System.out.println("Provided app package name not in dataset.  Make the sure the app package name is spelled correctly.");
				System.exit(1);
			}
			
			
		} catch(Exception e){
			e.printStackTrace();
		}
	}
	
	static boolean addApp(String currapp, ArrayList<String>appnames){
		
		if(appnames.contains(currapp)){ //if we have this app in our dataset
			
		}
		return false;
	}

}
